from pathlib import Path

BASE_DIR = Path(__file__).resolve().parent.parent

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = 'supersecretkey'

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

ALLOWED_HOSTS = ['*']

BOM_CONFIG = {
    'mouser_api_key': 'secretkey',
    'admin_dashboard': {
        'enable_autocomplete': False,
        'page_size': 50,
    }
}

# google GoogleOAuth
SOCIAL_AUTH_GOOGLE_OAUTH2_KEY = 'secretkey'
SOCIAL_AUTH_GOOGLE_OAUTH2_SECRET = 'secretkey'

# Database
# https://docs.djangoproject.com/en/1.10/ref/settings/#databases

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': BASE_DIR / 'db.sqlite3',
    }
}

FIXER_ACCESS_KEY = 'secretkey from fixer.io' # for exchange rate conversions


# Generated by Django 2.1.5 on 2019-02-23 23:53

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('bom', '0013_auto_20190222_1631'),
    ]

    operations = [
        migrations.RenameField(
            model_name='partchangehistory',
            old_name='part_id',
            new_name='part',
        ),
    ]



# Generated by Django 3.2.4 on 2021-11-23 01:57

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bom', '0042_auto_20210720_2137'),
    ]

    operations = [
        migrations.AlterField(
            model_name='partrevision',
            name='value_units',
            field=models.CharField(blank=True, choices=[('', '-----'), ('Ohms', 'Ω'), ('mOhms', 'mΩ'), ('kOhms', 'kΩ'), ('MOhms', 'MΩ'), ('F', 'F'), ('pF', 'pF'), ('nF', 'nF'), ('uF', 'μF'), ('V', 'V'), ('uV', 'μV'), ('mV', 'mV'), ('A', 'A'), ('uA', 'μA'), ('mA', 'mA'), ('C', '°C'), ('F', '°F'), ('H', 'H'), ('nH', 'nH'), ('mH', 'mH'), ('uH', 'μH'), ('Hz', 'Hz'), ('kHz', 'kHz'), ('MHz', 'MHz'), ('GHz', 'GHz'), ('Other', 'Other')], default=None, max_length=5, null=True),
        ),
        migrations.AlterField(
            model_name='subpart',
            name='count',
            field=models.FloatField(default=1, validators=[django.core.validators.MinValueValidator(0)]),
        ),
    ]



import logging
import os
from pathlib import Path

from django.utils.log import DEFAULT_LOGGING

logger = logging.getLogger(__name__)
BASE_DIR = Path(__file__).resolve().parent.parent

try:
    from .local_settings import *
except ImportError:
    logger.warning("local_settings.py not found. Using default settings.")
    pass

BOM_CONFIG = {}
BOM_CONFIG_DEFAULT = {
    'base_template': 'base.html',
    'mouser_api_key': None,
    'standalone_mode': True,
    'admin_dashboard': {
        'enable_autocomplete': True,
        'page_size': 50,
    }
}
BOM_ORGANIZATION_MODEL = 'bom.Organization'
BOM_USER_META_MODEL = 'bom.UserMeta'

# Apply custom settings over defaults
bom_config_new = BOM_CONFIG_DEFAULT.copy()
bom_config_new.update(BOM_CONFIG)
BOM_CONFIG = bom_config_new


# --------------------------------------------------------------------------
# APPLICATION DEFINITION
# --------------------------------------------------------------------------

INSTALLED_APPS = [
    # Custom Apps first
    'bom.apps.BomConfig',

    # Django contrib apps
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',

    # Third-party apps
    'materializecssform',
    'social_django',
    'djmoney',
    'djmoney.contrib.exchange',
]

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    'social_django.middleware.SocialAuthExceptionMiddleware',
]

ROOT_URLCONF = 'bom.urls'
WSGI_APPLICATION = 'bom.wsgi.application'
DEFAULT_AUTO_FIELD = 'django.db.models.BigAutoField'


# --------------------------------------------------------------------------
# TEMPLATES
# --------------------------------------------------------------------------

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        # Use pathlib syntax for cleaner path joining
        'DIRS': [BASE_DIR / 'bom' / 'templates' / 'bom'],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
                'django.template.context_processors.media',
                'social_django.context_processors.backends',
                'social_django.context_processors.login_redirect',
                'bom.context_processors.bom_config',
            ],
        },
    },
]


# --------------------------------------------------------------------------
# AUTHENTICATION & SECURITY
# --------------------------------------------------------------------------

AUTHENTICATION_BACKENDS = (
    'social_core.backends.google.GoogleOAuth2',
    'django.contrib.auth.backends.ModelBackend',
)

# Password validation - kept as is

AUTH_PASSWORD_VALIDATORS = [
    {'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',},
    {'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',},
    {'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',},
    {'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',},
]

# Social Auth Settings - kept as is
SOCIAL_AUTH_GOOGLE_OAUTH2_SCOPE = ['email', 'profile', 'https://www.googleapis.com/auth/drive', ]
SOCIAL_AUTH_GOOGLE_OAUTH2_AUTH_EXTRA_ARGUMENTS = {
    'access_type': 'offline',
    'approval_prompt': 'force'
}

SOCIAL_AUTH_PIPELINE = (
    'social_core.pipeline.social_auth.social_details',
    'social_core.pipeline.social_auth.social_uid',
    'social_core.pipeline.social_auth.social_user',
    'social_core.pipeline.user.get_username',
    'social_core.pipeline.social_auth.associate_by_email',
    'social_core.pipeline.user.create_user',
    'social_core.pipeline.social_auth.associate_user',
    'social_core.pipeline.social_auth.load_extra_data',
    'social_core.pipeline.user.user_details',
    'bom.third_party_apis.google_drive.initialize_parent',
)

SOCIAL_AUTH_DISCONNECT_PIPELINE = (
    'social_core.pipeline.disconnect.allowed_to_disconnect',
    'bom.third_party_apis.google_drive.uninitialize_parent',
    'social_core.pipeline.disconnect.get_entries',
    'social_core.pipeline.disconnect.revoke_tokens',
    'social_core.pipeline.disconnect.disconnect',
)


# --------------------------------------------------------------------------
# I18N & TIME
# --------------------------------------------------------------------------

LANGUAGE_CODE = 'en-us'
TIME_ZONE = 'UTC'
USE_I18N = True
USE_L10N = True # Deprecated in Django 4.0, but harmless for now
USE_TZ = True


# --------------------------------------------------------------------------
# FILE STORAGE (Static and Media)
# --------------------------------------------------------------------------

STATIC_URL = '/static/'
STATIC_ROOT = BASE_DIR / 'static'

MEDIA_URL = '/media/'
MEDIA_ROOT = BASE_DIR / 'media'

# Use the Django 4.2+ STORAGES setting
STORAGES = {
    # Default is FileSystemStorage, configured by STATIC_ROOT/MEDIA_ROOT
    "default": {
        "BACKEND": "django.core.files.storage.FileSystemStorage",
    },
    "staticfiles": {
        "BACKEND": "django.contrib.staticfiles.storage.StaticFilesStorage",
    },
}


# --------------------------------------------------------------------------
# URLS & REDIRECTS
# --------------------------------------------------------------------------

LOGIN_URL = '/login/'
LOGOUT_URL = '/logout/'

LOGIN_REDIRECT_URL = '/'
LOGOUT_REDIRECT_URL = '/'

SOCIAL_AUTH_LOGIN_REDIRECT_URL = '/settings?tab_anchor=file'
SOCIAL_AUTH_DISCONNECT_REDIRECT_URL = '/settings?tab_anchor=file'
SOCIAL_AUTH_LOGIN_ERROR_URL = '/'

# Custom login url for BOM_LOGIN (kept for compatibility)
BOM_LOGIN_URL = None


# --------------------------------------------------------------------------
# DJMONEY CONFIG
# --------------------------------------------------------------------------

CURRENCY_DECIMAL_PLACES = 4
EXCHANGE_BACKEND = 'djmoney.contrib.exchange.backends.FixerBackend'


# --------------------------------------------------------------------------
# LOGGING
# --------------------------------------------------------------------------

# Set DEBUG to False here if not defined in local_settings
DEBUG = locals().get('DEBUG', False)
LOG_FILE_PATH = '/var/log/indabom/django.log' if not DEBUG else BASE_DIR / 'bom.log'

LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'handlers': {
        'mail_admins': {
            'class': 'django.utils.log.AdminEmailHandler',
            'level': 'ERROR',
            'include_html': True,
        },
        'logfile': {
            'class': 'logging.handlers.WatchedFileHandler',
            'filename': LOG_FILE_PATH,
            'formatter': 'timestamp', # Define a simple formatter
        },
        'console': {
            'class': 'logging.StreamHandler',
            'formatter': 'timestamp',
        },
    },
    'formatters': {
        'timestamp': {
            'format': "[%(asctime)s] %(levelname)s [%(name)s.%(funcName)s:%(lineno)d] %(message)s"
        },
    },
    'loggers': {
        # Catchall logger
        '': {
            'handlers': ['console', 'logfile'],
            'level': 'INFO',
        },
        # Django logging
        'django': {
            'handlers': ['logfile'],
            'level': 'ERROR',
            'propagate': False,
        },
        'django.request': {
            'handlers': ['mail_admins', 'logfile'],
            'level': 'ERROR',
            'propagate': False,
        },
        # django-bom app
        'bom': {
            'handlers': ['logfile', 'console'],
            'level': 'INFO',
            'propagate': False
        },
    },
}


# Generated by Django 2.2.9 on 2019-12-31 16:30

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bom', '0028_partrevision_displayable_synopsis'),
    ]

    operations = [
        migrations.AlterField(
            model_name='partrevision',
            name='current_rating_units',
            field=models.CharField(blank=True, choices=[('', '-----'), ('A', 'A'), ('uA', 'μV'), ('mA', 'mA'), ('kA', 'kA'), ('MA', 'MA'), ('Other', 'Other')], default=None, max_length=5, null=True),
        ),
        migrations.AlterField(
            model_name='partrevision',
            name='voltage_rating_units',
            field=models.CharField(blank=True, choices=[('', '-----'), ('V', 'V'), ('uV', 'μV'), ('mV', 'mV'), ('kV', 'kV'), ('MV', 'MV'), ('Other', 'Other')], default=None, max_length=5, null=True),
        ),
    ]



# Generated by Django 2.2.9 on 2020-03-03 05:38

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('bom', '0035_auto_20200303_0111'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='partclass',
            unique_together={('code', 'organization')},
        ),
    ]



# Generated by Django 5.2.8 on 2026-01-04 00:59

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bom', '0050_alter_organization_options'),
        migrations.swappable_dependency(settings.BOM_ORGANIZATION_MODEL),
    ]

    operations = [
        migrations.AlterField(
            model_name='manufacturer',
            name='organization',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.BOM_ORGANIZATION_MODEL),
        ),
        migrations.AlterField(
            model_name='part',
            name='organization',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.BOM_ORGANIZATION_MODEL),
        ),
        migrations.AlterField(
            model_name='partclass',
            name='organization',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.BOM_ORGANIZATION_MODEL),
        ),
        migrations.AlterField(
            model_name='seller',
            name='organization',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.BOM_ORGANIZATION_MODEL),
        ),
        migrations.AlterField(
            model_name='usermeta',
            name='organization',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.BOM_ORGANIZATION_MODEL),
        ),
    ]



# Generated by Django 2.2.9 on 2020-02-22 03:59

import django.core.validators
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('bom', '0033_auto_20200203_0618'),
    ]

    operations = [
        migrations.AlterField(
            model_name='part',
            name='number_class',
            field=models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='number_class', to='bom.PartClass'),
        ),
        migrations.AlterField(
            model_name='part',
            name='number_item',
            field=models.CharField(blank=True, default=None, max_length=128, validators=[django.core.validators.RegexValidator('^[0-9a-zA-Z]*$', 'Only alphanumeric characters are allowed.')]),
        ),
        migrations.AlterField(
            model_name='part',
            name='number_variation',
            field=models.CharField(blank=True, default=None, max_length=2, null=True, validators=[django.core.validators.RegexValidator('^[0-9a-zA-Z]*$', 'Only alphanumeric characters are allowed.')]),
        ),
    ]



# Generated by Django 5.2.8 on 2025-12-16 18:43

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('bom', '0049_alter_assembly_id_alter_assemblysubparts_id_and_more'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='organization',
            options={'permissions': (('manage_members', 'Can manage organization members'),)},
        ),
    ]



from django.conf import settings
from django.contrib import admin
from django.contrib.auth import get_user_model
from django.contrib.auth.admin import UserAdmin

from .models import (
    Assembly,
    Manufacturer,
    ManufacturerPart,
    Part,
    PartClass,
    PartRevision,
    Seller,
    SellerPart,
    Subpart,
    get_organization_model,
    get_user_meta_model
)

User = get_user_model()
UserMeta = get_user_meta_model()
Organization = get_organization_model()

class UserMetaInline(admin.TabularInline):
    model = UserMeta
    verbose_name = 'BOM User Meta'
    raw_id_fields = ('organization',)
    can_delete = False


class OrganizationAdmin(admin.ModelAdmin):
    list_display = ('name',)


class SubpartInline(admin.TabularInline):
    model = Subpart
    fk_name = 'part_revision'
    raw_id_fields = ('part_revision',)
    # readonly_fields = ('get_full_part_number', )
    #
    # def get_full_part_number(self, obj):
    #     return obj.assembly_subpart.part.full_part_number()
    # get_full_part_number.short_description = 'PartNumber'


class SellerAdmin(admin.ModelAdmin):
    list_display = ('name',)


class SellerPartAdmin(admin.ModelAdmin):
    list_display = (
        'manufacturer_part',
        'seller',
        'seller_part_number',
        'minimum_order_quantity',
        'minimum_pack_quantity',
        'unit_cost',
        'lead_time_days',
        'nre_cost',
        'ncnr')


class SellerPartAdminInline(admin.TabularInline):
    model = SellerPart
    raw_id_fields = ('seller', 'manufacturer_part',)


class ManufacturerPartAdmin(admin.ModelAdmin):
    list_display = (
        'manufacturer_part_number',
        'manufacturer',
        'part',)
    raw_id_fields = ('manufacturer', 'part',)
    inlines = [
        SellerPartAdminInline,
    ]


class ManufacturerPartAdminInline(admin.TabularInline):
    model = ManufacturerPart
    raw_id_fields = ('part', 'manufacturer',)


class PartClassAdmin(admin.ModelAdmin):
    list_display = ('code', 'name', 'comment',)


class PartRevisionAdminInline(admin.TabularInline):
    model = PartRevision
    raw_id_fields = ('assembly',)


class PartAdmin(admin.ModelAdmin):
    ordering = ('organization', 'number_class__code', 'number_item', 'number_variation')
    readonly_fields = ('get_full_part_number',)
    list_display = (
        'organization',
        'get_full_part_number',
    )
    raw_id_fields = ('number_class', 'primary_manufacturer_part',)
    inlines = [
        ManufacturerPartAdminInline,
    ]

    def get_full_part_number(self, obj):
        return obj.full_part_number()

    get_full_part_number.short_description = 'PartNumber'
    get_full_part_number.admin_order_field = 'number_class__part_number'


class PartRevisionAdmin(admin.ModelAdmin):
    list_display = ('part', 'revision', 'description', 'get_assembly_size', 'timestamp',)
    raw_id_fields = ('assembly',)
    readonly_fields = ('timestamp',)

    def get_assembly_size(self, obj):
        return None if obj.assembly is None else obj.assembly.subparts.count()

    get_assembly_size.short_description = 'AssemblySize'


class ManufacturerAdmin(admin.ModelAdmin):
    list_display = ('name', 'organization',)


class SubpartAdmin(admin.ModelAdmin):
    list_display = ('part_revision', 'count', 'reference',)


class SubpartsInline(admin.TabularInline):
    model = Assembly.subparts.through
    raw_id_fields = ('subpart',)


class AssemblyAdmin(admin.ModelAdmin):
    list_display = ('id',)
    exclude = ('subparts',)
    inlines = [
        SubpartsInline,
    ]


if settings.BOM_USER_META_MODEL == 'bom.UserMeta':
    current_user_admin = admin.site._registry.get(User)

    if current_user_admin:
        user_admin_class = current_user_admin.__class__
        existing_inlines = list(user_admin_class.inlines or [])
        if UserMetaInline not in existing_inlines:
            existing_inlines.append(UserMetaInline)
            user_admin_class.inlines = existing_inlines
    else:
        class BomUserAdmin(UserAdmin):
            inlines = [UserMetaInline]


        admin.site.register(User, BomUserAdmin)

if settings.BOM_ORGANIZATION_MODEL == 'bom.Organization':
    from .models import Organization

    admin.site.register(Organization, OrganizationAdmin)

admin.site.register(Seller, SellerAdmin)
admin.site.register(SellerPart, SellerPartAdmin)
admin.site.register(ManufacturerPart, ManufacturerPartAdmin)
admin.site.register(PartClass, PartClassAdmin)
admin.site.register(Part, PartAdmin)
admin.site.register(PartRevision, PartRevisionAdmin)
admin.site.register(Manufacturer, ManufacturerAdmin)
admin.site.register(Assembly, AssemblyAdmin)
admin.site.register(Subpart, SubpartAdmin)



# Generated by Django 3.1 on 2020-09-29 23:15

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('bom', '0038_auto_20200422_0504'),
    ]

    operations = [
        migrations.AlterField(
            model_name='manufacturerpart',
            name='manufacturer',
            field=models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.CASCADE, to='bom.manufacturer'),
        ),
        migrations.AlterField(
            model_name='organization',
            name='owner',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL),
        ),
        migrations.AlterField(
            model_name='part',
            name='number_class',
            field=models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='number_class', to='bom.partclass'),
        ),
        migrations.AlterField(
            model_name='partrevision',
            name='assembly',
            field=models.ForeignKey(default=None, null=True, on_delete=django.db.models.deletion.CASCADE, to='bom.assembly'),
        ),
        migrations.AlterField(
            model_name='usermeta',
            name='organization',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='bom.organization'),
        ),
    ]



# Generated by Django 2.2.8 on 2020-01-04 13:52

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bom', '0030_auto_20200101_2253'),
    ]

    operations = [
        migrations.AlterField(
            model_name='partrevision',
            name='height_units',
            field=models.CharField(blank=True, choices=[('', '-----'), ('mil', 'mil'), ('in', 'in'), ('ft', 'ft'), ('yd', 'yd'), ('km', 'km'), ('m', 'm'), ('cm', 'cm'), ('mm', 'mm'), ('um', 'μm'), ('nm', 'nm'), ('Other', 'Other')], default=None, max_length=5, null=True),
        ),
        migrations.AlterField(
            model_name='partrevision',
            name='length_units',
            field=models.CharField(blank=True, choices=[('', '-----'), ('mil', 'mil'), ('in', 'in'), ('ft', 'ft'), ('yd', 'yd'), ('km', 'km'), ('m', 'm'), ('cm', 'cm'), ('mm', 'mm'), ('um', 'μm'), ('nm', 'nm'), ('Other', 'Other')], default=None, max_length=5, null=True),
        ),
        migrations.AlterField(
            model_name='partrevision',
            name='value_units',
            field=models.CharField(blank=True, choices=[('', '-----'), ('Ohms', 'Ω'), ('mOhms', 'mΩ'), ('kOhms', 'kΩ'), ('F', 'F'), ('pF', 'pF'), ('nF', 'nF'), ('uF', 'μF'), ('V', 'V'), ('uV', 'μV'), ('mV', 'mV'), ('A', 'A'), ('uA', 'μA'), ('mA', 'mA'), ('C', '°C'), ('F', '°F'), ('H', 'H'), ('mH', 'mH'), ('uH', 'μH'), ('Hz', 'Hz'), ('kHz', 'kHz'), ('MHz', 'MHz'), ('GHz', 'GHz'), ('Other', 'Other')], default=None, max_length=5, null=True),
        ),
        migrations.AlterField(
            model_name='partrevision',
            name='wavelength_units',
            field=models.CharField(blank=True, choices=[('', '-----'), ('km', 'km'), ('m', 'm'), ('cm', 'cm'), ('mm', 'mm'), ('um', 'μm'), ('nm', 'nm'), ('A', 'Å'), ('Other', 'Other')], default=None, max_length=5, null=True),
        ),
        migrations.AlterField(
            model_name='partrevision',
            name='width_units',
            field=models.CharField(blank=True, choices=[('', '-----'), ('mil', 'mil'), ('in', 'in'), ('ft', 'ft'), ('yd', 'yd'), ('km', 'km'), ('m', 'm'), ('cm', 'cm'), ('mm', 'mm'), ('um', 'μm'), ('nm', 'nm'), ('Other', 'Other')], default=None, max_length=5, null=True),
        ),
    ]
